//------------------------------------------------------------------------------------------------

//
// File: polygon.cpp
// Author: Roman Lysecky
// Date Created: 28 November 2011
// 
// Description: Declaration of singly-linked list of LineSegment's representing a polygon.
// 
// Copyright: Roman Lysecky, Electrical and Computer Engineering, University of Arizona, 2011.
// 
// 

//------------------------------------------------------------------------------------------------

#include <iostream>
#include <list>
#include "linesegment.h"
#include "polygon.h"

//------------------------------------------------------------------------------------------------

using namespace std;

//------------------------------------------------------------------------------------------------

// Polygon constructor: initialize polygon to empty list 
Polygon::Polygon()
{
}

//------------------------------------------------------------------------------------------------

// 
// Polygon destructor
//
Polygon::~Polygon()
{
}

//------------------------------------------------------------------------------------------------

// 
// inserts new line segment specified by (x1,y1), (x2, y2) at the end of the list  
//
void 
Polygon::insert_segment(char *id, float x1, float y1, float x2, float y2)
{   
    segments.push_back(LineSegment(id, x1, y1, x2, y2));
}

//------------------------------------------------------------------------------------------------

/* returns the number of line segments within the polygon */
int 
Polygon::num_segments()
{
    return segments.size();
}

//------------------------------------------------------------------------------------------------

/* 
 * searches the polygon to find a segment that start (or ends) at the end of the provided 
 * current segment,returning a pointer to the connected segment if it exists. Otherwise, returns NULL
 *
 * If the connected segment ends (x2, y2) at the end point of the current segment, the start and
 * end of that segment will be swapped. This ensures the algorithm within is_valid_polygon()
 * always follows from the start (x1, y1) to the end (x2, y2) of every connected line segment.
 */
LineSegment * 
Polygon::find_next_connected_segment(LineSegment* fromSegment) 
{
    list<LineSegment>::iterator toSegment;
    float tempVal;
    
    toSegment = segments.begin();
    
    /* while we have not reached the end of the list of line segments */
    while (toSegment != segments.end() ) {
        
        /* only conisder those segments that are not the current segment */
        if( &*toSegment != fromSegment ) {
            
            /* if end (x2, y2) of from segment is equal to start of the to segment */
            if(fromSegment->x2 == toSegment->x1 &&
               fromSegment->y2 == toSegment->y1 ) {
                
                /* return the found segment */
                return &*toSegment;
            }
            /* if end (x2, y2) of from segment is equal to end of the to segment */
            else if (fromSegment->x2 == toSegment->x2 &&
                     fromSegment->y2 == toSegment->y2 ) {
                
                /* sawp the start and end of the to line segment */
                tempVal = toSegment->x2;
                toSegment->x2 = toSegment->x1;
                toSegment->x1 = tempVal;
                tempVal = toSegment->y2;
                toSegment->y2 = toSegment->y1;
                toSegment->y1 = tempVal;
                
                /* return found segment */
                return &*toSegment;
            }
        }
        toSegment++;
    }
    
    /* did not find a conneced segment */
    return 0;
}

//------------------------------------------------------------------------------------------------

/* 
 * checks to determine if the specified polygon is a valid closed form polygon in which all segments 
 * are connected to another line segment
 *
 * NOTE: There is still a potential error with this implementation.
 */
bool 
Polygon::is_valid()
{
    LineSegment* segment;
    unsigned int segments_followed;
    
    /* start with first line segment in polygon */
    segment = &segments.front();
    segments_followed = 0;
    
    /* 
     * for each line segment found, search for another line segment that starts or ends
     * at the end point (x2, y2) of the current segment 
     *
     * continue as long as we have found another connected segment, we have not returned the
     * head of the list, and we have not followed more than the number of segments that are in
     * the polygon (i.e. make sure we don't get stuck in a cycle)
     */
    do {
        segment = find_next_connected_segment(segment);
        
        /* if the segment is not found, the polygon is invalid */
        if( segment == 0 ) return false;
        
        /* if segment was found, increase the count of segments we have followed */
        segments_followed++;
        
    } while (segment != &segments.front() && segments_followed != segments.size());
    
    /* 
     * if we returned to the head in exactly the number of segments that exist in the 
     * polygon, we have a valid polygon shape 
     */
    if(segment == &segments.front()  && segments_followed == segments.size()) return true;
    
    return false;
}

//------------------------------------------------------------------------------------------------

ostream& 
operator<< (ostream &os, Polygon &polygon)
{
    // Use an iterator to traverse the list
    list<LineSegment>::iterator iter;
    
    // Start at beginning of list.
    iter=polygon.segments.begin();
    
    // While we have not reached the end of the list
    while(iter != polygon.segments.end()) {
        if (iter != polygon.segments.begin() ) cout << ", ";
        
        // To access the list element itself, we need to dereference the iterator
        // (just like you would derefence a pointer).
        cout << *iter;        
        
        // Move to the next element
        iter++;
    }
    
    return os;
}

//------------------------------------------------------------------------------------------------

